#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sensors


def main():
    sensors.init()
    try:
        for chip in sensors.iter_detected_chips():
            print chip
            print 'Adapter:', chip.adapter_name
            for feature in chip:
                print '%s (%r): %.1f' % (
                    feature.name, feature.label, feature.get_value()
                )
                for subfeature in feature:
                    print '  %s: %.1f' % (
                        subfeature.name, subfeature.get_value()
                    )
            print
    finally:
        sensors.cleanup()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from ctypes import *

import common
import stdc
from common import *

__all__ = ['API_VERSION', 'DEFAULT_CONFIG_FILENAME', 'iter_detected_chips']

API_VERSION = 3
common.DEFAULT_CONFIG_FILENAME = DEFAULT_CONFIG_FILENAME = '/etc/sensors.conf'


class Feature(Structure):
    _fields_ = [
        ('number', c_int),
        ('name', c_char_p),
        ('mapping', c_int),
        ('compute_mapping', c_int),
        ('mode', c_int),
    ]
    
    NO_MAPPING = -1
    
    def __repr__(self):
        return '<%s number=%d name=%r mapping=%d compute_mapping=%d mode=%d' % (
            self.__class__.__name__,
            self.number,
            self.name,
            self.mapping,
            self.compute_mapping,
            self.mode
        )
    
    def __iter__(self):
        return self.chip._iter_features(self.number)
    
    @property
    def label(self):
        result_p = c_char_p()
        _get_label(self.chip, self.number, byref(result_p))
        result = result_p.value
        stdc.free(result_p)
        return result
    
    def get_value(self):
        result = c_double()
        _get_feature(self.chip, self.number, byref(result))
        return result.value

FEATURE_P = POINTER(Feature)


class Chip(Structure):
    # 
    # TODO Implement a `__str__()` method.
    # TODO Move common stuff into `AbstractChip` class.
    # 
    _fields_ = [
        ('prefix', c_char_p),
        ('bus', c_int),
        ('addr', c_int),
        ('busname', c_char_p),
    ]
    
    def __new__(cls, *args):
        result = super(Chip, cls).__new__(cls)
        if args:
            _parse_chip_name(args[0], byref(result))
        return result
    
    def __init__(self, *_args):
        Structure.__init__(self)
    
    def __repr__(self):
        return '<%s prefix=%r bus=%r addr=%r busname=%r>' % (
            (
                self.__class__.__name__,
                self.prefix,
                self.bus,
                self.addr,
                self.busname
            )
        )
    
    def __iter__(self):
        return self._iter_features()
    
    @property
    def has_wildcards(self):
        return bool(_chip_name_has_wildcards(self))
    
    @property
    def adapter_name(self):
        return _get_adapter_name(self.bus)
    
    def match(self, other):
        return bool(_match_chip(self, other))
    
    def _iter_features(self, parent=Feature.NO_MAPPING):
        nr1, nr2 = c_int(0), c_int(0)
        while True:
            result_p = _get_all_features(self, byref(nr1), byref(nr2))
            if not result_p:
                break
            result = result_p.contents
            if result.mapping == parent:
                result.chip = self
                result.subfeatures = dict()
                yield result

CHIP_P = POINTER(Chip)


# 
# TODO Implement a handler for at least `sensors_parse_error`.
# 

_parse_chip_name = SENSORS_LIB.sensors_parse_chip_name
_parse_chip_name.argtypes = [c_char_p, CHIP_P]
_parse_chip_name.restype = c_int
_parse_chip_name.errcheck = _error_check

_match_chip = SENSORS_LIB.sensors_match_chip
_match_chip.argtypes = [Chip, Chip]
_match_chip.restype = c_int

_chip_name_has_wildcards = SENSORS_LIB.sensors_chip_name_has_wildcards
_chip_name_has_wildcards.argtypes = [Chip]
_chip_name_has_wildcards.restype = c_int

_get_adapter_name = SENSORS_LIB.sensors_get_adapter_name
_get_adapter_name.argtypes = [c_int]
_get_adapter_name.restype = c_char_p

_get_label = SENSORS_LIB.sensors_get_label
_get_label.argtypes = [Chip, c_int, POINTER(c_char_p)]
_get_label.restype = c_int
_parse_chip_name.errcheck = _error_check

# 
# TODO sensors_get_ignored()
# 

_get_feature = SENSORS_LIB.sensors_get_feature
_get_feature.argtypes = [Chip, c_int, POINTER(c_double)]
_get_feature.restype = c_int
_parse_chip_name.errcheck = _error_check

# 
# TODO sensors_set_feature()
# TODO sensors_do_chip_sets()
# TODO sensors_do_all_sets()    (common function)
# 

_get_detected_chips = SENSORS_LIB.sensors_get_detected_chips
_get_detected_chips.argtypes = [POINTER(c_int)]
_get_detected_chips.restype = CHIP_P

_get_all_features = SENSORS_LIB.sensors_get_all_features
_get_all_features.argtypes = [Chip, POINTER(c_int), POINTER(c_int)]
_get_all_features.restype = FEATURE_P


def iter_detected_chips(chip_name='*-*'):
    chip = Chip(chip_name)
    number = c_int(0)
    while True:
        result_p = _get_detected_chips(byref(number))
        if not result_p:
            break
        result = result_p.contents
        # 
        # TODO Add ignore check.
        # 
        if chip.match(result):
            yield result



# -*- coding: utf-8 -*-
import os
from ctypes import *
from ctypes.util import find_library

import stdc

__all__ = [
    'SENSORS_LIB', 'VERSION', 'MAJOR_VERSION', 'SensorsError', '_error_check',
    'init', 'cleanup'
]

LIB_FILENAME = os.environ.get('SENSORS_LIB') or find_library('sensors')
SENSORS_LIB = CDLL(LIB_FILENAME)
VERSION = c_char_p.in_dll(SENSORS_LIB, 'libsensors_version').value
MAJOR_VERSION = int(VERSION.split('.', 1)[0])


class SensorsError(Exception):
    def __init__(self, message, error_number=None):
        Exception.__init__(self, message)
        self.error_number = error_number


def _error_check(result, _func, _arguments):
    if result < 0:
        raise SensorsError(_strerror(result), result)

_strerror = SENSORS_LIB.sensors_strerror
_strerror.argtypes = [c_int]
_strerror.restype = c_char_p

_init = SENSORS_LIB.sensors_init
_init.argtypes = [c_void_p]
_init.restype = c_int
_init.errcheck = _error_check

cleanup = SENSORS_LIB.sensors_cleanup
cleanup.argtypes = None
cleanup.restype = None


def init(config_filename=None):
    if config_filename is None:
        config_filename = DEFAULT_CONFIG_FILENAME
    file_p = stdc.fopen(config_filename, 'r')
    if file_p is None:
        error_number = get_errno()
        raise OSError(error_number, os.strerror(error_number), config_filename)
    try:
        _init(file_p)
    finally:
        stdc.fclose(file_p)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from ctypes import *

import common
import stdc
from common import *

__all__ = ['API_VERSION', 'DEFAULT_CONFIG_FILENAME', 'iter_detected_chips']

API_VERSION = 4
common.DEFAULT_CONFIG_FILENAME = DEFAULT_CONFIG_FILENAME = '/etc/sensors3.conf'


class Subfeature(Structure):
    _fields_ = [
        ('name', c_char_p),
        ('number', c_int),
        ('type', c_int),
        ('mapping', c_int),
        ('flags', c_uint),
    ]
    
    def __repr__(self):
        return '<%s name=%r number=%d type=%d mapping=%d flags=%08x>' % (
            self.__class__.__name__,
            self.name,
            self.number,
            self.type,
            self.mapping,
            self.flags
        )
    
    def get_value(self):
        result = c_double()
        _get_value(byref(self.parent.chip), self.number, byref(result))
        return result.value

SUBFEATURE_P = POINTER(Subfeature)


class Feature(Structure):
    _fields_ = [
        ('name', c_char_p),
        ('number', c_int),
        ('type', c_int),
        ('_first_subfeature', c_int),
        ('_padding1', c_int),
    ]
    
    def __repr__(self):
        return '<%s name=%r number=%r type=%r>' % (
            self.__class__.__name__,
            self.name,
            self.number,
            self.type
        )
    
    def __iter__(self):
        number = c_int(0)
        while True:
            result_p = _get_all_subfeatures(
                byref(self.chip),
                byref(self),
                byref(number)
            )
            if not result_p:
                break
            result = result_p.contents
            result.chip = self.chip
            result.parent = self
            yield result
    
    @property
    def label(self):
        # 
        # TODO Maybe this is a memory leak!
        # 
        return _get_label(byref(self.chip), byref(self))
    
    def get_value(self):
        # 
        # TODO Is the first always the correct one for all feature types?
        # 
        return iter(self).next().get_value()

FEATURE_P = POINTER(Feature)


class Bus(Structure):
    TYPE_ANY = -1
    NR_ANY = -1
    
    _fields_ = [
        ('type', c_short),
        ('nr', c_short),
    ]
    
    def __str__(self):
        return (
            '*' if self.type == self.TYPE_ANY
                else _get_adapter_name(byref(self))
        )
    
    def __repr__(self):
        return '%s(%r, %r)' % (self.__class__.__name__, self.type, self.nr)
    
    @property
    def has_wildcards(self):
        return self.type == self.TYPE_ANY or self.nr == self.NR_ANY

BUS_P = POINTER(Bus)


class Chip(Structure):
    # 
    # TODO Move common stuff into `AbstractChip` class.
    # 
    _fields_ = [
        ('prefix', c_char_p),
        ('bus', Bus),
        ('addr', c_int),
        ('path', c_char_p),
    ]
    
    PREFIX_ANY = None
    ADDR_ANY = -1
    
    def __new__(cls, *args):
        result = super(Chip, cls).__new__(cls)
        if args:
            _parse_chip_name(args[0], byref(result))
        return result
        
    def __init__(self, *_args):
        Structure.__init__(self)
        # 
        # Need to bind the following to the instance so it is available in
        #  `__del__()` when the interpreter shuts down.
        # 
        self._free_chip_name = _free_chip_name
        self.byref = byref
    
    def __del__(self):
        if self._b_needsfree_:
            self._free_chip_name(self.byref(self))
    
    def __repr__(self):
        return '<%s prefix=%r bus=%r addr=%r path=%r>' % (
            (
                self.__class__.__name__,
                self.prefix,
                self.bus,
                self.addr,
                self.path
            )
        )
    
    def __str__(self):
        buffer_size = 200
        result = create_string_buffer(buffer_size)
        used = _snprintf_chip_name(result, len(result), byref(self))
        assert used < buffer_size
        return result.value
    
    def __iter__(self):
        number = c_int(0)
        while True:
            result_p = _get_features(byref(self), byref(number))
            if not result_p:
                break
            result = result_p.contents
            result.chip = self
            yield result
    
    @property
    def adapter_name(self):
        return str(self.bus)
    
    @property
    def has_wildcards(self):
        return (
            self.prefix == self.PREFIX_ANY
            or self.addr == self.ADDR_ANY
            or self.bus.has_wildcards
        )

CHIP_P = POINTER(Chip)


_parse_chip_name = SENSORS_LIB.sensors_parse_chip_name
_parse_chip_name.argtypes = [c_char_p, CHIP_P]
_parse_chip_name.restype = c_int
_parse_chip_name.errcheck = _error_check

_free_chip_name = SENSORS_LIB.sensors_free_chip_name
_free_chip_name.argtypes = [CHIP_P]
_free_chip_name.restype = None

_snprintf_chip_name = SENSORS_LIB.sensors_snprintf_chip_name
_snprintf_chip_name.argtypes = [c_char_p, c_size_t, CHIP_P]
_snprintf_chip_name.restype = c_int
_snprintf_chip_name.errcheck = _error_check

_get_adapter_name = SENSORS_LIB.sensors_get_adapter_name
_get_adapter_name.argtypes = [BUS_P]
_get_adapter_name.restype = c_char_p

_get_label = SENSORS_LIB.sensors_get_label
_get_label.argtypes = [CHIP_P, FEATURE_P]
_get_label.restype = c_char_p

_get_value = SENSORS_LIB.sensors_get_value
_get_value.argtypes = [CHIP_P, c_int, POINTER(c_double)]
_get_value.restype = c_int
_get_value.errcheck = _error_check

# 
# TODO sensors_set_value()
# TODO sensors_do_chip_sets()
# 

_get_detected_chips = SENSORS_LIB.sensors_get_detected_chips
_get_detected_chips.argtypes = [CHIP_P, POINTER(c_int)]
_get_detected_chips.restype = CHIP_P

_get_features = SENSORS_LIB.sensors_get_features
_get_features.argtypes = [CHIP_P, POINTER(c_int)]
_get_features.restype = FEATURE_P

_get_all_subfeatures = SENSORS_LIB.sensors_get_all_subfeatures
_get_all_subfeatures.argtypes = [CHIP_P, FEATURE_P, POINTER(c_int)]
_get_all_subfeatures.restype = SUBFEATURE_P

# 
# TODO sensors_get_subfeature() ?
# 

def iter_detected_chips(chip_name='*-*'):
    chip = Chip(chip_name)
    number = c_int(0)
    while True:
        result = _get_detected_chips(byref(chip), byref(number))
        if not result:
            break
        yield result.contents



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from distutils.core import setup
import sensors

setup(
    name='PySensors',
    version=sensors.__version__,
    author=sensors.__author__,
    author_email=sensors.__contact__,
    packages=['sensors'],
    #scripts=[],
    url='http://pypi.python.org/pypi/PySensors/',
    #download_url='',
    license=sensors.__license__,
    description='Python bindings to libsensors (via ctypes)',
    long_description=open('README.rst').read(),
    keywords=['sensors', 'hardware', 'monitoring'],
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Topic :: System',
        'Topic :: System :: Hardware',
        'Topic :: System :: Monitoring',
    ]
)



# -*- coding: utf-8 -*-
from ctypes import *
from ctypes.util import find_library

STDC_LIB = CDLL(find_library('c'), use_errno=True)

fopen = STDC_LIB.fopen
fopen.argtypes = [c_char_p, c_char_p]
fopen.restype = c_void_p

fclose = STDC_LIB.fclose
fclose.argtypes = [c_void_p]
fclose.restype = c_int

free = STDC_LIB.free
free.argtypes = [c_void_p]
free.restype = None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from common import *

if MAJOR_VERSION < 3:
    from api3 import *
else:
    from api4 import *


__version__ = '0.0.2'
__date__ = '2011-10-04'
__author__ = "Marc 'BlackJack' Rintsch"
__contact__ = 'marc@rintsch.de'
# 
# Due to lm-sensors 2.x using this license.
# 
__license__ = 'GPL v2'
